let nodemailer = require("nodemailer");
const PushBullet = require("pushbullet");

// Used to summon users into the IRC via email to <user>@tilde.team
module.exports = class Summon {
    constructor(client, from, to, incantation) {
        this.name = "Summon";
        this.client = client;
        this.summoner = from;
        this.channel = to;
        this.incantation = incantation;
        this.spell = "kunvoki";
        this.cheatCode = "summon";
        this.casted = false;
        this.varsUsed = 2;
        this.locked = false;
        this.transporter = nodemailer.createTransport({
            sendmail: true,
            newline: "unix",
            path: "/usr/sbin/sendmail"
        });
        this.pusher = new PushBullet(client.config.api.pushbullet);
    }

    test() {
        if (this.incantation === this.spell) {
            // Only the author can use these spells
            if (this.locked && this.summoner !== this.client.config.author) {
                const say = "Blasfemo! That spell is forbidden to you."
                this.client.say(this.channel, say);
                return false;
            }

            this.casted = true;
            return this.cast.apply(this, arguments);
        }
        
        return { say: false };
    }

    cast(user, ...reason) {
        let response = {
            say: false,
            debug: {},
            content: ""
        };

        if (!user) {
            response.say = true;
            response.content = `${from}, malsagxulo!. You must tell me`;
            response.content = `${response.content} who to summon.`;
            return response;
        }

        if (!reason) {
            reason = "<none>";
        } else {
            reason = reason.join(" ");
        }

        const subject = "You have been summoned!";
        const botName = this.client.config.botName;
        let text = `My bot, ${botName}, received`;
        text = `${text} a summoning request for you`;
        text = `${text} from ${this.summoner}`;
        text = `${text} in channel ${this.channel}`;
        text = `${text} for reason: ${reason}`;

        let email = `${user}@tilde.team`;
        if (user.startsWith("@")) {
            email = email.slice(1);
        }

        this.transporter.sendMail({
            from: `${botName}@tilde.team`,
            to: email,
            subject: subject,
            text: text
        }, (err, info) => {
            // DEBUG
            // console.log("Summon::email", info);
            response.debug.email = info;
        });

        // I obvious don't know everyone's Pushbullet API
        if (user === this.client.config.author) {
            const self = this;
            let sent = false;
            self.pusher.devices().then(function (res) {
                const devices = res.devices;
                devices.forEach((device) => {
                    if (sent) return false;
                    if (device.nickname === self.client.config.device) {
                        const iden = device.iden;
                        self.pusher.note(iden, subject, text, (err, info) => {
                            // DEBUG
                            // console.log("Summon::pusher", info);
                            response.debug.pusher = info;
                        });
                        sent = true;
                    }
                });
            });
        }

        const success = `Superba ${this.summoner}! You have summoned`;

        response.say = true;
        response.content = `${success} ${user}!`;

        return response;
    }
}